@@ -13,9 +13,11 @@ use futures::StreamExt;
 use log::LevelFilter;
 use md5::digest::generic_array::GenericArray;
 use md5::{Digest, Md5};
+use sodiumoxide::hex;
 use sqlx::sqlite::SqliteConnectOptions;
 use sqlx::{ConnectOptions, Sqlite, SqlitePool, Transaction};
-use tokio::fs::{remove_file, rename};
+use tokio::fs::{remove_file, rename, File};
+use tokio::join;
 use tokio::sync::mpsc::{channel, Receiver, Sender};
 use tokio_stream::wrappers::ReceiverStream;
 use tracing::{debug, error, info, instrument, warn};
@@ -184,27 +186,7 @@ async fn db_listener(
             for item in items {
                 debug!("deleting file due to exceeding cache size");
                 size_freed += item.size as u64;
-                tokio::spawn(async move {
-                    let key = item.id;
-                    if let Err(e) = remove_file(key.clone()).await {
-                        match e.kind() {
-                            std::io::ErrorKind::NotFound => {
-                                let hash = Md5Hash(*GenericArray::from_slice(key.as_bytes()));
-                                let path: PathBuf = hash.into();
-                                if let Err(e) = remove_file(&path).await {
-                                    warn!(
-                                        "Failed to delete file `{}` from cache: {}",
-                                        path.to_string_lossy(),
-                                        e
-                                    );
-                                }
-                            }
-                            _ => {
-                                warn!("Failed to delete file `{}` from cache: {}", &key, e);
-                            }
-                        }
-                    }
-                });
+                tokio::spawn(remove_file_handler(item.id));
             }
 
             cache.disk_cur_size.fetch_sub(size_freed, Ordering::Release);
@@ -212,29 +194,50 @@ async fn db_listener(
     }
 }
 
-#[instrument(level = "debug", skip(transaction))]
-async fn handle_db_get(entry: &Path, transaction: &mut Transaction<'_, Sqlite>) {
-    let hash = if let Ok(hash) = Md5Hash::try_from(entry) {
-        hash
+/// Returns if a file was successfully deleted.
+async fn remove_file_handler(key: String) -> bool {
+    let error = if let Err(e) = remove_file(&key).await {
+        e
     } else {
-        error!(
-            "Failed to derive hash from entry, dropping message: {}",
-            entry.to_string_lossy()
-        );
-        return;
+        return true;
     };
 
-    let hash_str = hash.to_hex_string();
+    if error.kind() != std::io::ErrorKind::NotFound {
+        warn!("Failed to delete file `{}` from cache: {}", &key, error);
+        return false;
+    }
+
+    if let Ok(bytes) = hex::decode(&key) {
+        if bytes.len() != 16 {
+            warn!("Failed to delete file `{}`; invalid hash size.", &key);
+            return false;
+        }
+
+        let hash = Md5Hash(*GenericArray::from_slice(&bytes));
+        let path: PathBuf = hash.into();
+        if let Err(e) = remove_file(&path).await {
+            warn!(
+                "Failed to delete file `{}` from cache: {}",
+                path.to_string_lossy(),
+                e
+            );
+            false
+        } else {
+            true
+        }
+    } else {
+        warn!("Failed to delete file `{}`; not a md5hash.", &key);
+        false
+    }
+}
+
+#[instrument(level = "debug", skip(transaction))]
+async fn handle_db_get(entry: &Path, transaction: &mut Transaction<'_, Sqlite>) {
     let key = entry.as_os_str().to_str();
     let now = chrono::Utc::now();
-    let query = sqlx::query!(
-        "update Images set accessed = ? where id = ? or id = ?",
-        now,
-        key,
-        hash_str
-    )
-    .execute(transaction)
-    .await;
+    let query = sqlx::query!("update Images set accessed = ? where id = ?", now, key)
+        .execute(transaction)
+        .await;
     if let Err(e) = query {
         warn!("Failed to update timestamp in db for {:?}: {}", key, e);
     }
@@ -297,7 +300,7 @@ impl TryFrom<&Path> for Md5Hash {
 
 impl From<Md5Hash> for PathBuf {
     fn from(hash: Md5Hash) -> Self {
-        let hex_value = dbg!(hash.to_hex_string());
+        let hex_value = hash.to_hex_string();
         let path = hex_value[0..3]
             .chars()
             .rev()
@@ -319,14 +322,35 @@ impl Cache for DiskCache {
     ) -> Option<Result<(CacheStream, ImageMetadata), CacheError>> {
         let channel = self.db_update_channel_sender.clone();
 
-        // TODO: Check legacy path as well
-
         let path = Arc::new(self.disk_path.clone().join(PathBuf::from(key)));
         let path_0 = Arc::clone(&path);
 
-        tokio::spawn(async move { channel.send(DbMessage::Get(path_0)).await });
+        let legacy_path = Md5Hash::try_from(path_0.as_path())
+            .map(PathBuf::from)
+            .map(|path| self.disk_path.clone().join(path))
+            .map(Arc::new);
 
-        super::fs::read_file_from_path(&path).await.map(|res| {
+        // Get file and path of first existing location path
+        let (file, path) = if let Ok(legacy_path) = legacy_path {
+            let maybe_files = join!(
+                File::open(legacy_path.as_path()),
+                File::open(path.as_path()),
+            );
+            match maybe_files {
+                (Ok(f), _) => Some((f, legacy_path)),
+                (_, Ok(f)) => Some((f, path)),
+                _ => return None,
+            }
+        } else {
+            File::open(path.as_path())
+                .await
+                .ok()
+                .map(|file| (file, path))
+        }?;
+
+        tokio::spawn(async move { channel.send(DbMessage::Get(path)).await });
+
+        super::fs::read_file(file).await.map(|res| {
             res.map(|(stream, _, metadata)| (stream, metadata))
                 .map_err(|_| CacheError::DecryptionFailure)
         })
@@ -378,6 +402,61 @@ impl CallbackCache for DiskCache {
     }
 }
 
+#[cfg(test)]
+mod remove_file_handler {
+
+    use std::error::Error;
+
+    use tempfile::tempdir;
+    use tokio::fs::{create_dir_all, remove_dir_all};
+
+    use super::*;
+
+    #[tokio::test]
+    async fn should_not_panic_on_invalid_path() {
+        assert!(!remove_file_handler("/this/is/a/non-existent/path/".to_string()).await);
+    }
+
+    #[tokio::test]
+    async fn should_not_panic_on_invalid_hash() {
+        assert!(!remove_file_handler("68b329da9893e34099c7d8ad5cb9c940".to_string()).await);
+    }
+
+    #[tokio::test]
+    async fn should_not_panic_on_malicious_hashes() {
+        assert!(!remove_file_handler("68b329da9893e34".to_string()).await);
+        assert!(
+            !remove_file_handler("68b329da9893e34099c7d8ad5cb9c940aaaaaaaaaaaaaaaaaa".to_string())
+                .await
+        );
+    }
+
+    #[tokio::test]
+    async fn should_delete_existing_file() -> Result<(), Box<dyn Error>> {
+        let temp_dir = tempdir()?;
+        let mut dir_path = temp_dir.path().to_path_buf();
+        dir_path.push("abc123.png");
+
+        // create a file, it can be empty
+        File::create(&dir_path).await?;
+
+        assert!(remove_file_handler(dir_path.to_string_lossy().into_owned()).await);
+
+        Ok(())
+    }
+
+    #[tokio::test]
+    async fn should_delete_existing_hash() -> Result<(), Box<dyn Error>> {
+        create_dir_all("b/8/6").await?;
+        File::create("b/8/6/68b329da9893e34099c7d8ad5cb9c900").await?;
+
+        assert!(remove_file_handler("68b329da9893e34099c7d8ad5cb9c900".to_string()).await);
+
+        remove_dir_all("b").await?;
+        Ok(())
+    }
+}
+
 #[cfg(test)]
 mod disk_cache {
     use std::error::Error;
