@@ -2,9 +2,10 @@ use reqwest::StatusCode;
 use serde::Serialize;
 use tracing::{info, warn};
 
+use crate::client::HTTP_CLIENT;
 use crate::config::ClientSecret;
 
-const CONTROL_CENTER_STOP_URL: &str = "https://api.mangadex.network/ping";
+const CONTROL_CENTER_STOP_URL: &str = "https://api.mangadex.network/stop";
 
 #[derive(Serialize)]
 struct StopRequest<'a> {
@@ -12,11 +13,10 @@ struct StopRequest<'a> {
 }
 
 pub async fn send_stop(secret: &ClientSecret) {
-    let request = StopRequest { secret };
-    let client = reqwest::Client::new();
-    match client
+    match HTTP_CLIENT
+        .inner()
         .post(CONTROL_CENTER_STOP_URL)
-        .json(&request)
+        .json(&StopRequest { secret })
         .send()
         .await
     {
